library out_dmx;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the DMX Output Module for Lightning.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

{$IMAGEBASE $411A0000}

uses
  SysUtils,
  Windows,
  ShellAPI,
  Math,
  StdCtrls,
  Spin,
  Graphics,
  Dialogs,
  common in '..\common.pas';

{$R *.RES}

const
  channels = 32;

type
  TDetails = class(TObject)
  public
    previousInput: Byte;
    oldStatus: array[1..channels] of Integer;
    firstDMXChannel: Integer;
    gbDetails: TGroupBox;
    lFirstDMXChannel: TLabel;
    seFirstDMXChannel: TSpinEdit;
    bUpdate: TButton;
    procedure bUpdateClick(Sender: TObject);
  end;

var
  wnd: HWND;
  m:   UINT;

procedure TDetails.bUpdateClick(Sender: TObject);
begin
  firstDMXChannel := seFirstDMXChannel.Value - 1;
end;

function init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  Result := return('ODMX|Hardware', name, nameLength);
end;

procedure finish; cdecl;
begin
  //
end;

function create(controlHandle: THandle): PLightningModule; cdecl;
var
  myDetails: TDetails;
  i:         Integer;
begin
  New(Result);
  with Result^ do
  begin
    handle        := controlHandle;
    title         := 'DMX';
    settings      := True;
    width         := -1;
    height        := -1;
    inputTop      := -1;
    infoTop       := -1;
    outputTop     := -1;
    inputs        := channels;
    outputs       := 0;
    GetMem(inputTooltip, maxLength);
    StrCopy(inputTooltip, '');
    for i := 1 to channels do
      StrCat(inputTooltip, PChar('Channel ' + IntToStr(i) + ' value|'));
    outputTooltip := '';
    myDetails     := TDetails.Create;
    details       := myDetails;
    reserved      := nil;
  end;
  with myDetails do
  begin
    firstDMXChannel  := 0;
    previousInput := channels + 1;
    for i := 1 to channels do
      oldStatus[i] := -1;
  end;
  m := RegisterWindowMessage('DMXLINK');
  wnd := FindWindow('TfDMXLink', 'DMXLink');
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
begin
  FreeMem(lightningModule^.inputTooltip);
  TDetails(lightningModule^.details).Free;
  Dispose(lightningModule);
end;

function action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
var
  details: TDetails;
  level: TLevel;
  channel: Word;
begin
  Result  := False;
  details := TDetails(lightningModule^.details);
  if input < details.previousInput then
    wnd := FindWindow('TfDMXLink', 'DMXLink');
  level := spectrumAverage(bridge, fftMin, fftMax);
  if (details.oldStatus[input] = -1) or (level <> details.oldStatus[input]) then
  begin
    details.oldStatus[input] := level;
    channel := details.firstDMXChannel + input - 1;
    if channel < 512 then
    begin
      PostMessage(
        wnd,
        m,
        MAKEWPARAM(
          0,
          0
        ),
        MAKELPARAM(
          MakeWord(level div 20, LOBYTE(channel)),
          MakeWord(HIBYTE(channel), 0)
        )
      );
    end;
  end;
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
begin
  with TDetails(lightningModule.details) do
  begin
    gbDetails := TGroupBox.Create(nil);
    with gbDetails do
    begin
      Top     := detailsTop;
      Left    := detailsLeft;
      Width   := detailsWidth;
      Height  := detailsHeight;
      Ctl3D   := True;
      Color   := clBtnFace;
      Caption := 'DMX Output Settings';
    end;
    lFirstDMXChannel := TLabel.Create(gbDetails);
    with lFirstDMXChannel do
    begin
      AutoSize := True;
      Top      := 20;
      Left     := 8;
      Caption  := 'First DMX output channel:';
      Parent   := gbDetails;
    end;
    seFirstDMXChannel := TSpinEdit.Create(gbDetails);
    with seFirstDMXChannel do
    begin
      Top       := 40;
      Left      := 8;
      MinValue  := 1;
      MaxValue  := 513 - channels;
      Increment := 1;
      Value     := firstDMXChannel + 1;
      Parent    := gbDetails;
    end;
    bUpdate := TButton.Create(gbDetails);
    with bUpdate do
    begin
      Top     := 40;
      Left    := 140;
      Width   := 120;
      Height  := seFirstDMXChannel.Height;
      Caption := 'Update';
      OnClick := bUpdateClick;
      Parent  := gbDetails;
    end;
    gbDetails.ParentWindow := handle;
  end;
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  with TDetails(lightningModule.details) do
  begin
    seFirstDMXChannel.Free;
    lFirstDMXChannel.Free;
    gbDetails.Free;
  end;
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
var
  bString: string;
begin
  with TDetails(lightningModule.details) do
  begin
    bString := settings;
    firstDMXChannel := StrToIntDef(parse(bString, ','), 0);
  end;
end;

function save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
begin
  with TDetails(lightningModule^.details) do
    Result := return(IntToStr(firstDMXChannel), settings, settingsLength);
end;

exports
  init, finish, create, destroy, action, show, hide, open, save;

begin
end.
